#include <Bounce.h>
#include <Shifter.h>

//-------------------------------------------------------//
//Define PINS numbers//
//-------------------------------------------------------//

//TACT SWITCHES CONFIG
byte PBPIN[15] = { //Pin adresses of the tact switches
  1,//Switch 0
  2,//Switch 1
  3,//Switch 2
  8,//Switch 3
  9,//Switch 4

  11,//Switch 5
  12,//Switch 6
  4,//Switch 7
  6,//Switch 8
  0,//Switch 9

  5,//Switch 10
  7,//Switch 11
  10,//Switch 12
  16,//Switch 13
  17
};//Switch 14

//LEDs shifts setup
#define SHIFT_OUT_DATA 23 //Serial PIN
#define SHIFT_OUT_LATCH 22 //Latch PIN
#define SHIFT_OUT_CLOCK 21 //Clock PIN
#define SHIFT_OUT_REGISTERS 4 //# shift registers
Shifter shifter(SHIFT_OUT_DATA, SHIFT_OUT_LATCH, SHIFT_OUT_CLOCK, SHIFT_OUT_REGISTERS); //Instanciate the shifter library object

//An index to match led numbers on PCB with led on 595 chain
byte LEDSEQ[8] = {
  //TOGGLE LEDS
  2,    //11
  3,    //0
  4,    //1
  5,    //2
  7,    //8

  10,    //16
  12,   //10

  15,    //19

};

byte LEDPINS[29] = {
  //TOGGLE LEDS
  3,    //0
  4,    //1
  5,    //2
  16,   //3
  17,   //4
  21,   //5
  22,   //6
  //2-WAY SWITCHES
  6,    //7
  7,    //8
  11,   //9
  12,   //10
  //3-WAY SWITCHES
  0,    //11
  1,    //12
  2,    //13

  8,    //14
  9,    //15
  10,    //16

  13,    //17
  14,    //18
  15,    //19

  18,    //20
  19,    //21
  20,    //22

  23,    //24
  24,    //25
  25,    //26

  26,    //27
  27,
  28
};

//Analog MUXES
//Configure MUX adress pins
byte SPINS[3] = {
  13, 14, 15
};

//Configure MUX OUT PIN
byte VOUT_ANA1 = 18; //read pin
byte VOUT_ANA2 = 19; //read pin
byte VOUT_ANA3 = 20; //read pin

//-------------------------------------------------------//
//DEBOUNCE TACT SWITCHES
//-------------------------------------------------------//
//Configure DEBOUNCE OBJECTS FOR SWITCHES with a debounce of 60ms
Bounce PBOUNCED[15] =
{
  Bounce((PBPIN[0]), 60),
  Bounce((PBPIN[1]), 60),
  Bounce((PBPIN[2]), 60),
  Bounce((PBPIN[3]), 60),
  Bounce((PBPIN[4]), 60),

  Bounce((PBPIN[5]), 60),
  Bounce((PBPIN[6]), 60),
  Bounce((PBPIN[7]), 60),
  Bounce((PBPIN[8]), 60),
  Bounce((PBPIN[9]), 60),

  Bounce((PBPIN[10]), 60),
  Bounce((PBPIN[11]), 60),
  Bounce((PBPIN[12]), 60),
  Bounce((PBPIN[13]), 60),
  Bounce((PBPIN[14]), 60)
};


//-------------------------------------------------------//
//Sequencer
//-------------------------------------------------------//
float tempo = 128;
float tempo_buffer = 128;
float old_tempo = 128;
float clock_divider = 8; //0.25=1/4 //
float old_clock_divider = 8;
float clock_divider_buffer = 8;
boolean new_divider = 0;
float manual_clock = 0;
byte shift = 4; // 4 starts the sequencer at notes 48 C3 when the fader is all the way down
byte resolution = 12; //12=1 octave along the fader //24 = 2 octaves along the fader //60 = 5 octaves
byte seq_direction = 1;
long seq_direction_ratio = 1;
byte sequence_length = 16;
byte old_sequence_length = 16;
byte sequence_length_buffer = 16;
elapsedMillis blink_sequencer = 0;
boolean blink1=0;
boolean blink2=0;
boolean step_time_flag=0;

boolean seq_page = 0;
boolean seq_on = 0;
boolean midi_clock = 0;
byte midi_clock_counter = 0;
byte midi_clock_counter2 = 0;
byte beat_counter = 0  ;
boolean midi_clock_flag = 0;
byte CLOCK = 248;
byte START = 250;
byte CONTINUE = 251;
byte STOP = 252;
boolean SEQ_MODE = 0;
boolean SEQ_IN = 0;
boolean SEQ_OUT = 0;
boolean SEQ_SWITCH = 0;

elapsedMicros step_timer = 0;

byte step_counter = 255; //start @ 1111 1111, that the first counter++=0;
byte previous_step_counter = 255; //start @ 1111 1111, that the first counter++=0;


int Analog_resoluted[16] = {
  0, 1, 2, 3,
  4, 5, 6, 7,
  8, 9, 10, 11,
  12, 13, 14, 15
};

int Previous_Analog_resoluted[16] = {
  0, 1, 2, 3,
  4, 5, 6, 7,
  8, 9, 10, 11,
  12, 13, 14, 15
};

int seq_notes[16]  = {
  60, 61, 62, 63,
  64, 65, 66, 67,
  68, 69, 70, 71,
  72, 73, 74, 75
};

int seq_notes_buffer[16]  = {
  60, 61, 62, 63,
  64, 65, 66, 67,
  68, 69, 70, 71,
  72, 73, 74, 75
};

int step_index[8] = {
  9, 0, 1, 2,
  7, 10, 8, 11
};

boolean step_state[16] = {
  1, 1, 1, 1,
  1, 1, 1, 1,
  1, 1, 1, 1,
  1, 1, 1, 1
};

//-------------------------------------------------------//
//VARIABLES
//-------------------------------------------------------//

byte PBSTATUS[15] { //Store the binary
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0
};

//ARRAY FOR PUSHBUTTONS STATES
byte buttonState_PB[15]  = {
  //ON-OFF switches
  1,  //0 SQUARE
  1,  //1 TRIANGLE
  0,  //2 SUB
  1,  //3 CH1
  0,  //4 CH2

  //2-WAY switches
  0,  //5 ARP
  0,  //6 SYNC
  1,  //7 VCF_SW
  0,  //8 VCA_SW

  //3-WAY switches
  1,  //9 DCO_SW
  1,  //10 MST_SW
  0,  //11 LFO_SW
  0,  //12 PRT_SW
  0,  //13 MOD_SW
  0   //14 RNG_SW
};

byte PreviousbuttonState_PB[15]  = {
  1, 1, 0, 1,
  0, 0, 0, 1,
  0, 1, 1, 0,
  0, 0, 0
};

byte LastPreviousbuttonState_PB[15]  = {
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 0,
  0, 0, 0, 0, 2
};

byte AnalogThreshold = 8; //threshold
elapsedMillis timer = 0;
byte Smoothing = 30;
byte AnalogSamplingPeriod = 8;

//Debug mux pins
byte AnaIndex[24] = {
  2, 1, 0, 3, 10, 13, 11, 12,
  16, 15, 14, 17, 18, 21, 19, 20,
  6, 5, 4, 7, 22, 9, 23, 8
};

int Analog[24] = {
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0
};

byte Analog_Debug[24] = {
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 1, 1
};

int Previous_Analog[24] = {
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0
};

byte Analog_Mapped[24] = {
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0
};

byte Previous_Analog_Mapped[24] = {
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0
};

//MUX control array SFC_60_RevA_V1_8_no_presets
int muxChannel[8][3] = {
  {
    0, 0, 0
  }
  ,
  {
    1, 0, 0
  }
  ,
  {
    0, 1, 0
  }
  ,
  {
    1, 1, 0
  }
  ,
  {
    0, 0, 1
  }
  ,
  {
    1, 0, 1
  }
  ,
  {
    0, 1, 1
  }
  ,
  {
    1, 1, 1
  }
  ,
};

boolean InitFlag = 0;
int PressTime14 = 0;
boolean Pressed14 = 0;
boolean LongPressed14 = 0;
int PressTime13 = 250;
boolean Pressed13 = 0;
boolean LongPressed13 = 0;
int PressTime6 = 0;
boolean Pressed6 = 0;
boolean LongPressed6 = 0;
boolean LedState = 1;
boolean booting = 1;

// SETUP (pin config) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //
void setup() {

  Serial.begin(9600);

  //PB pin config
  for (int i = 0; i < 15; i++) {
    pinMode(PBPIN[i], INPUT_PULLUP);
  }

  //VOUT MUX PIN CONFIG
  pinMode(VOUT_ANA1, INPUT);
  pinMode(VOUT_ANA2, INPUT);
  pinMode(VOUT_ANA3, INPUT);

  //MUX adress PINS CONFIG
  for (int i = 0; i < 3; i++) {
    pinMode(SPINS[i], OUTPUT);
  }

  //RESET PINS CONFIG
  for (int i = 0; i < 3; i++) {
    digitalWrite(SPINS[i], LOW);
  }

  shifter.clear();

  Boot();

  //for midi clock
  usbMIDI.setHandleRealTimeSystem(RealTimeSystem);

}

// LOOPS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //
void loop() {

  //-------------------------------------------------------//
  //READING DIGITAL INPUTS//
  //-------------------------------------------------------//
  ReadDigital();

  //-------------------------------------------------------//
  //READING AND SMOOTHING ANALOG//
  //-------------------------------------------------------//
  ReadAnalog();

  if (!SEQ_MODE) {
    //-------------------------------------------------------//
    //TOGGLE SWITCHES AND DIGITAL MIDI
    //-------------------------------------------------------//
    MidiDigital();

    //-------------------------------------------------------//
    //SAMPLING, TESTING THRESHOLD AND SENDS MIDI//
    //-------------------------------------------------------//
    MidiAnalog();

    //-------------------------------------------------------//
    //RECALL INIT//
    //-------------------------------------------------------//
    Recall();
  }

  //-------------------------------------------------------//
  //LEDs CONTROL//
  //-------------------------------------------------------//
  Leds();

  step_time_flag = 0;

  Sequencer();


  //for midi clock
  usbMIDI.read();

  InitFlag = 0;
  booting = 0;

}



